<?php
/**
 * Classe spxStats 
 *
 * @version 1.0
 * @package PLX
 * @author    Stephane F modifi par je-evrard
 * http://wiki.pluxml.org/index.php?page=Statistiques%3A+Affichage+des+visiteurs+en+ligne+et+pages+vues
 **/
 
# vrification de la prsence du dossier stats
if(!is_dir(PLX_PLUGINS.'/stats')) {
	mkdir(PLX_PLUGINS.'/stats',0755,true);
}
 
 
 
class spxStats {

	private static $instance = null;
    var $filename = null; # nom du fichier contenant les statistiques
    var $aStats = array(); # donnes statistiques
    var $sessionTime = 15; # dure d'une visite en minute vant expiration

    /**
     * Constructeur qui initialise certaines variables de classe
     * et qui lance les traitements de prise en compte des statistiques
     *
     * @param    null
     * @return    null
     * @author    Stephane F
     **/    
    function spxStats() {
    
        $this->filename = PLX_ROOT.'plugins/stats/stats.xml'; # emplacement du fichier des stats
    
        $this->_getStatiques($this->filename); # rcuperation des stats
        $this->_cleanVisits(); # suppression des anciens visiteurs 
        $this->_addVisitor(); # ajoute le nouveau visiteur et gre les compteurs
        $this->_editStats($this->filename); # sauvegarde des stats
    }
	
	/**
	 * Mthode qui se charger de crer le Singleton spxStats
	 *
	 * @return	objet			return une instance de la classe spxStats
	 * @author	je-evrard
	 **/
	public static function getInstance(){
		if (!isset(self::$instance))
			self::$instance = new spxStats();
		return self::$instance;
	}
    
    #------------------
    # Mthodes prives
    #------------------
    
    /**
     * Mthode qui parse le fichier des statistiques et qui alimente
     * le tableau aStats
     *
     * @param    null
     * @return    null
     * @author    Stephane F
     **/
    function _getStatiques() {

        if(!is_readable($this->filename)) return;
            
        # Mise en place du parseur XML
        $data = implode('',file($this->filename));
        $parser = xml_parser_create(PLX_CHARSET);
        xml_parser_set_option($parser,XML_OPTION_CASE_FOLDING,0);
        xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE,0);
        xml_parse_into_struct($parser,$data,$values,$iTags);
        xml_parser_free($parser);
        # Recuperation des valeurs de nos champs XML
        $this->aStats['views_today'] = array( // total de pages vues aujourd'hui
                'date'         => (isset($values[ $iTags['views_today'][0] ]['attributes']['date'])?$values[ $iTags['views_today'][0] ]['attributes']['date']:date('Ymd')),
                'counter'    => (isset($values[ $iTags['views_today'][0] ]['value'])?intval($values[ $iTags['views_today'][0] ]['value']):0)
            );
        $this->aStats['views_total'] = isset($values[ $iTags['views_total'][0] ]['value'])?intval($values[ $iTags['views_total'][0] ]['value']):0; // total de pages vues
        $this->aStats['visitors'] = isset($values[ $iTags['visitors'][0] ]['value'])?intval($values[ $iTags['visitors'][0] ]['value']):0; // nombre total de visiteurs 
        
        # On verifie qu'il existe des tags "online"
        if(isset($iTags['online'])) {
            # On compte le nombre de tags "online"
            $nb = sizeof($iTags['online']);
            # On boucle sur $nb
            for($i = 0; $i < $nb; $i++) {
                $this->aStats['online'][ $values[ $iTags['online'][$i] ]['attributes']['ip'] ] = $values[ $iTags['online'][$i] ]['value'];
            }
        }
    }
    
    /**
     * Mthode qui sauvegarde les statistiques dans le fichier xml
     * le tableau aStats
     *
     * @param    null
     * @return    null
     * @author    Stephane F
     **/    
    function _editStats() {

        # Dbut du fichier XML
        $xml = "<?xml version='1.0' encoding='".PLX_CHARSET."'?>\n";
        $xml .= "<document>\n";
        $xml .= "\t<views_today date=\"".$this->aStats['views_today']['date']."\">".$this->aStats['views_today']['counter']."</views_today>\n";        
        $xml .= "\t<views_total>".$this->aStats['views_total']."</views_total>\n";
        $xml .= "\t<visitors>".$this->aStats['visitors']."</visitors>\n";
        if ($this->aStats['online']) {
            foreach($this->aStats['online'] as $ip => $time) {
                $xml .= "\t<online ip=\"".$ip."\">".$time."</online>\n";
            }
        }
        $xml .= "</document>";
        # Sauvegarde du fichier
        plxUtils::write($xml, $this->filename);
    }
    
    /**
     * Mthode qui supprime les anciens visiteurs si la dure de la visite
     * est expire
     *
     * @param    null
     * @return    null
     * @author    Stephane F
     **/    
    function _cleanVisits() {
    
        if ($this->aStats['online']) {
            foreach($this->aStats['online'] as $ip => $time) {
                if(time() - $time >= $this->sessionTime * 60)
                    unset($this->aStats['online'][$ip]); # suppresion de l'ancien visiteur
            }
        }
        
    }
    
    /**
     * Mthode qui ajoute un nouveau visiteur et qui incrmente les diffrents
     * compteurs de statistiques
     *
     * @param    null
     * @return    null
     * @author    Stephane F
     **/    
    function _addVisitor() {
    
        if (isset($this->aStats['online'][plxUtils::getIp()])) {
            if (time() - $this->aStats['online'][plxUtils::getIp()] >= $this->sessionTime * 60) {
                $this->aStats['online'][plxUtils::getIp()] = time(); 
                $this->aStats['visitors']++; 
            }
        }
        else {
            $this->aStats['online'][plxUtils::getIp()] = time(); 
            $this->aStats['visitors']++; 
        }

        # nombre de visiteurs du jour
        if ($this->aStats['views_today']['date'] == date('Ymd'))
            $this->aStats['views_today']['counter']++;
        else {
            $this->aStats['views_today']['date'] = date('Ymd');
            $this->aStats['views_today']['counter']=1;
        }
        
        # nombre total de pages vues
        $this->aStats['views_total']++; 

    }
    
    #-------------------
    # Mthodes publiques
    #-------------------
    
    /**
     * Mthode qui affiche le compteur du nombre total de visiteurs
     *
     * @param    null
     * @return    stdout
     * @author    Stephane F
     **/    
    function TotalVisitors() {
        return  $this->aStats['visitors'];
    }
    
    /**
     * Mthode qui affiche le compteur du visiteurs en ligne
     *
     * @param    null
     * @return    stdout
     * @author    Stephane F
     **/    
    function OnlineVisitors() {
      global $nbtemp;
        return  sizeof($this->aStats['online']);
    }
	
	 /**
     * Ecris 's' s'il y a plusieurs visiteurs
     *
     * @param    null
     * @return    stdout
     * @author    Ludo_17 et Amaury G
     **/    
    function Pluriel() {
        if(sizeof($this->aStats['online']) > 1)
			return  's';
    }
	    
    /**
     * Mthode qui affiche le compteur du nombre de pages vues aujourd'hui
     *
     * @param    null
     * @return    stdout
     * @author    Stephane F
     **/
    function TodayViews() {
        return $this->aStats['views_today']['counter'];
    }
    
    /**
     * Mthode qui affiche le compteur du nombre total de pages vues
     *
     * @param    null
     * @return    stdout
     * @author    Stephane F
     **/    
    function TotalViews() {
        return $this->aStats['views_total'];
    }   


	
}


#-----------------------------------------------
# cration d'une instance de la classe spxStats
#-----------------------------------------------
// $spxStats = new spxStats=>getInstance();


/* frontend
<h2>Statistiques</a></h2>
    <ul>
        <li>Total des pages vues: <?php $spxStats->TotalViews() ?></li>
        <li>Pages vues aujourd'hui:  <?php $spxStats->TodayViews() ?></li>
        <li><?php echo "Visiteur" ?><?php $spxStats->Pluriel() ?> <?php echo " connect"; ?><?php $spxStats->Pluriel() ?>: <?php $spxStats->OnlineVisitors() ?></li>
        <li>Nombre de visiteurs: <?php $spxStats->TotalVisitors() ?></li>
    </ul>


*/


/* frontend personne connect uniquement
<?php if(isset($_SESSION['user'])) : ?>
    <h2>Statistiques</a></h2>
    <ul>
        <li>Total des pages vues: <?php $spxStats->TotalViews() ?></li>
        <li>Pages vues aujourd'hui:  <?php $spxStats->TodayViews() ?></li>
        <li><?php echo "Visiteur" ?><?php $spxStats->Pluriel() ?> <?php echo " connect"; ?><?php $spxStats->Pluriel() ?>: <?php $spxStats->OnlineVisitors() ?></li>
        <li>Nombre de visiteurs: <?php $spxStats->TotalVisitors() ?></li>
    </ul>
<?php endif; ?>

*/

?>